﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Collections;

namespace decimal_bias_namespace
{
    public partial class MainWindow : Form
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        WriteNextLine WriteLine = new WriteNextLine();
        string[] strInputFileNames = new string[100];
        bool boolInputFilesSelected = false;
        bool boolOutputFileSelected = false;
        int intPrecisionDecimals = 5;
        int intLevels = 0;
        int intDecimals = 0;
        
        private void SetDecimalBiasOrder()
        {
            if (radioButton1.Checked)
            {
                intLevels = 10; intDecimals = 1;
            };
            if (radioButton2.Checked)
            {
                intLevels = 100; intDecimals = 2;
            };
            if (radioButton3.Checked)
            {
                intLevels = 1000; intDecimals = 3;
            };
            if (radioButton4.Checked)
            {
                intLevels = 10000; intDecimals = 4;
            };
        }

        private void RadioBtnChanged()
        {
            SetDecimalBiasOrder();

            switch (intDecimals)
            {
                case 1:
                    status.Text = "decimal levels 0 to 9  will be examined";
                break;

                case 2:
                    status.Text = "decimal levels 00 to 99  will be examined";
                break;

                case 3:
                    status.Text = "decimal levels 000 to 999  will be examined";
                break;

                case 4:
                    status.Text = "decimal levels 0000 to 9999 will be examined";
                break;
            }
        }

        private void PerformStats()
        {
            if (boolInputFilesSelected && boolOutputFileSelected)
            {
                WriteLine.DeleteFileIfNoAppend();

                SetDecimalBiasOrder();
                int[] resultsArray = new int[intLevels];

                foreach (string strFileName in strInputFileNames)
                {
                    if (strFileName.Substring(strFileName.Length - 40, 1) == "\\")
                    {
                        input_file_processed_label.Text = strFileName;

                        try
                        {
                            StreamReader MyStreamReader = new StreamReader(strFileName);

                            int intLineNumber = 1;
                            string strThisLine = "";
                            while (MyStreamReader.EndOfStream != true)
                            {
                                strThisLine = MyStreamReader.ReadLine().Replace(".", "");
                                strThisLine = strThisLine.Substring(strThisLine.Length - intDecimals, intDecimals);
                                resultsArray[int.Parse(strThisLine)]++;
                                intLineNumber++;
                            }
                            intLineNumber--;

                            string strSymbolName = strFileName.Substring(strFileName.Length - 39, 6);
                            status.Text = "total lines read: " + intLineNumber.ToString() + ", last symbol: " + strSymbolName;

                            WriteLine.Write(strSymbolName);
                            WriteLine.Write("level,occurences,percent");

                            for (int intLevels_c = 0; intLevels_c < intLevels; intLevels_c++)
                            {
                                WriteLine.Write(
                                    intLevels_c.ToString()
                                    + ","
                                    + resultsArray[intLevels_c].ToString()
                                    + ","
                                    + Decimal.Round((decimal)resultsArray[intLevels_c] / (decimal)intLineNumber * 100, intPrecisionDecimals).ToString()
                                    );
                            }
                            WriteLine.Write("");
                        }
                        catch (Exception ex)
                        {
                            status.Text = "last error: " + ex.Message;
                            MessageBox.Show(ex.Message, "Error");
                        }
                    }
                    else
                    {
                        MessageBox.Show("filename not in a correct format: \r" + strFileName, "unknown filename format");
                        status.Text = "last error: wrong filename format";
                    }
                }
                MessageBox.Show("end of processing", "done");
            }
            else
                MessageBox.Show("please select both the input and output files first", "select the files");

        }

        private void run_the_stats_button_Click(object sender, EventArgs e)
        {
            PerformStats();
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            strInputFileNames = openFileDialog1.FileNames;
            boolInputFilesSelected = true;
            label5.Text = "yes";
            status.Text = "input file(s) selected successfully";
        }

        private void openToolStripButton_Click(object sender, EventArgs e)
        {
            openFileDialog1.FileName = "";
            openFileDialog1.ShowDialog();
        }

        private void saveToolStripButton_Click(object sender, EventArgs e)
        {
            saveFileDialog1.FileName = "Decimal_Bias_Stats.csv";
            saveFileDialog1.DefaultExt = ".csv";
            saveFileDialog1.ShowDialog();
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            WriteLine.strOutputFileName = saveFileDialog1.FileName;
            boolOutputFileSelected = true;
            output_filename_label.Text = WriteLine.strOutputFileName;
            status.Text = "output file selected successfully";
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            WriteLine.boolAppend = checkBox1.Checked;
            switch (checkBox1.Checked)
            {
                case false:
                    status.Text = "output file be deleted/created anew";
                break;
                
                case true:
                    status.Text = "results will be appended to the existing output file";
                break;
            }
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            RadioBtnChanged();
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            RadioBtnChanged();
        }

        private void radioButton3_CheckedChanged(object sender, EventArgs e)
        {
            RadioBtnChanged();
        }

        private void radioButton4_CheckedChanged(object sender, EventArgs e)
        {
            RadioBtnChanged();
        }
    }

    class WriteNextLine
    {
        public string strOutputFileName = "";
        public bool boolAppend = false;

        public void DeleteFileIfNoAppend()
        {
            if (!boolAppend)
            {
                File.Delete(strOutputFileName);
            }
        }

        public void Write(string input_string)
        {
            using (StreamWriter MyStreamWriter = new StreamWriter(strOutputFileName, true))
            {
                MyStreamWriter.WriteLine(input_string);
            }
        }
    }

}